/*
 * AIEngine a new generation network intrusion detection system.
 *
 * Copyright (C) 2013-2017  Luis Campo Giralte
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 *
 * Written by Luis Campo Giralte <luis.camp0.2009@gmail.com> 
 *
 */
#ifndef SRC_STACKTEST_H_
#define SRC_STACKTEST_H_

#include <string>
#include "Multiplexer.h"
#include "FlowForwarder.h"
#include "protocols/ethernet/EthernetProtocol.h"
#include "AnomalyManager.h"
#include "CacheManager.h"

using namespace aiengine;

struct StackTest {
        EthernetProtocolPtr eth;
        MultiplexerPtr mux_eth;
	SharedPointer<AnomalyManager> anomaly;
	SharedPointer<CacheManager> cache_mng;

        StackTest() {
                eth = EthernetProtocolPtr(new EthernetProtocol());
                mux_eth = MultiplexerPtr(new Multiplexer());

                // Configure the eth
                eth->setMultiplexer(mux_eth);
                mux_eth->setProtocol(static_cast<ProtocolPtr>(eth));
		mux_eth->setProtocolIdentifier(0);
                mux_eth->setHeaderSize(eth->getHeaderSize());
                mux_eth->addChecker(std::bind(&EthernetProtocol::ethernetChecker,eth,std::placeholders::_1));

		anomaly = SharedPointer<AnomalyManager>(new AnomalyManager());
		cache_mng = SharedPointer<CacheManager>(new CacheManager());
        }

        void inject(Packet &pkt) {
                mux_eth->setPacket(&pkt);
                eth->setHeader(mux_eth->getCurrentPacket()->getPayload());
                mux_eth->setNextProtocolIdentifier(eth->getEthernetType());
                mux_eth->forwardPacket(pkt);
        }

        ~StackTest() {}
};

#endif  // SRC_STACKTEST_H_
